@@ -411,7 +411,8 @@ class Thread(KernelObject):
         self.teb = None
         self.seh = SEH()
         self.tls = []
-        self.ctx = None
+        self.message_queue = []
+        self.ctx = self.emu.get_thread_context()
         self.fls = []
         self.suspend_count = 0
         self.token = Token(self.emu)
@@ -422,6 +423,12 @@ class Thread(KernelObject):
         self.write_back()
         self.process = None
 
+    def queue_message(self, msg):
+        """
+        Add a GUI message to the thread's message queue
+        """
+        self.message_queue.append(msg)
+
     def get_seh(self):
         return self.seh
 
@@ -695,8 +702,6 @@ class Process(KernelObject):
         self.peb.write_back()
 
     def init_peb(self, modules):
-        self.peb.object.ImageBaseAddress = self.base
-
         # Add an entry for each module in the module list
         for mod in modules:
             self.add_module_to_peb(mod)
