@@ -11,6 +11,8 @@ import speakeasy.windows.objman as objman
 from .. import api
 import hashlib
 
+SERVICE_STATUS_HANDLE_BASE = 0x1000
+
 
 class AdvApi32(api.ApiHandler):
     """
@@ -31,6 +33,7 @@ class AdvApi32(api.ApiHandler):
         self.win = adv32
         self.curr_rand = 0
         self.curr_handle = 0x2800
+        self.service_status_handle = SERVICE_STATUS_HANDLE_BASE
 
         super(AdvApi32, self).__get_hook_attrs__(self)
 
@@ -416,13 +419,69 @@ class AdvApi32(api.ApiHandler):
         ste = self.win.SERVICE_TABLE_ENTRY(emu.get_ptr_size())
         entry = self.mem_cast(ste, lpServiceStartTable)
 
-        # Get the service name
-        name = self.read_mem_string(entry.lpServiceName, cw) # noqa
+        argv[0] = "lpServiceStartTable=["
+
+        while (entry.lpServiceName != windefs.NULL or
+                entry.lpServiceProc != windefs.NULL):
+            # Get the service name
+            if entry.lpServiceName != windefs.NULL:
+                name = self.read_mem_string(entry.lpServiceName, cw) # noqa
+                argv[0] += " {{ lpServiceName={}".format(name)
+            else:
+                argv[0] += " { lpServiceName=NULL"
+            # Get the ServiceMain function
+            if entry.lpServiceProc != windefs.NULL:
+                service_main = entry.lpServiceProc
+                argv[0] += ", lpServiceProc={} }} ".format(hex(service_main))
+                handle, obj = self.create_thread(service_main, windefs.NULL,
+                                                 emu.get_current_process())
+            else:
+                argv[0] += ", lpServiceProc=NULL } "
+            # next entry
+            lpServiceStartTable += self.sizeof(ste)
+            ste = self.win.SERVICE_TABLE_ENTRY(emu.get_ptr_size())
+            entry = self.mem_cast(ste, lpServiceStartTable)
+
+        argv[0] += "]"
+
         rv = True
         emu.set_last_error(windefs.ERROR_SUCCESS)
 
         return rv
 
+    @apihook('RegisterServiceCtrlHandler', argc=2)
+    def RegisterServiceCtrlHandler(self, emu, argv, ctx={}):
+        '''
+        SERVICE_STATUS_HANDLE RegisterServiceCtrlHandlerA(
+            LPCSTR             lpServiceName,
+            LPHANDLER_FUNCTION lpHandlerProc
+            );
+        '''
+
+        lpServiceName, lpHandlerProc = argv
+
+        # dummy SERVICE_STATUS_HANDLE
+        self.service_status_handle += 1
+
+        emu.set_last_error(windefs.ERROR_SUCCESS)
+
+        return self.service_status_handle
+
+    @apihook('SetServiceStatus', argc=2)
+    def SetServiceStatus(self, emu, argv, ctx={}):
+        '''
+        BOOL SetServiceStatus(
+            SERVICE_STATUS_HANDLE hServiceStatus,
+            LPSERVICE_STATUS      lpServiceStatus
+            );
+        '''
+
+        hServiceStatus, lpServiceStatus = argv
+
+        emu.set_last_error(windefs.ERROR_SUCCESS)
+
+        return 0x1
+
     @apihook('OpenSCManager', argc=3)
     def OpenSCManager(self, emu, argv, ctx={}):
         '''
@@ -741,6 +800,9 @@ class AdvApi32(api.ApiHandler):
         if not info_len:
             rv = False
             emu.set_last_error(windefs.ERROR_INSUFFICIENT_BUFFER)
+            
+        if info_class == 20 and info and emu.get_user().get('is_admin', True):
+            self.mem_write(info, (1).to_bytes(4, 'little'))
 
         if ret_len:
             self.mem_write(ret_len, (4).to_bytes(4, 'little'))
